@@ -1,5 +1,6 @@
 /* bfd initialization stuff
-   Copyright (C) 1990-1991 Free Software Foundation, Inc.
+   Copyright 1990, 1991, 1992, 1993, 1994, 1995, 2003
+   Free Software Foundation, Inc.
    Written by Steve Chamberlain of Cygnus Support.
 
 This file is part of BFD, the Binary File Descriptor library.
@@ -16,16 +17,12 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #include "bfd.h"
 #include "sysdep.h"
 #include "libbfd.h"
 
-extern void DEFUN_VOID (bfd_section_init);
-
-static boolean initialized = false;
-
 /*
 SECTION
 	Initialization
@@ -38,42 +35,17 @@ FUNCTION
 	bfd_init
 
 SYNOPSIS
-	void bfd_init(void);
+	void bfd_init (void);
 
 DESCRIPTION
 	This routine must be called before any other BFD function to
 	initialize magical internal data structures.
 */
 
-void
-bfd_init ()
-{
-  if (initialized == false) {
-    initialized = true;
-
-    bfd_arch_init();
-  }
-}
-
-
-/*
-INTERNAL_FUNCTION
-	bfd_check_init
-
-SYNOPSIS
-	void bfd_check_init(void);
-
-DESCRIPTION
-	This routine is called before any other BFD function using
-	initialized data. It ensures that the structures have
-	been initialized.  Soon this function will go away, and the BFD
-	library will assume that <<bfd_init>> has been called.
-*/
+/* Actually, there is currently nothing for this function to do.
+   However, someday it may be needed, so keep it around.  */
 
 void
-bfd_check_init ()
+bfd_init (void)
 {
-  if (initialized == false) {
-    bfd_init();
-  }
 }
