@@ -1025,8 +1025,8 @@ avr_frame_prev_register (struct frame_info *next_frame,
         }
     }
   else
-    trad_frame_prev_register (next_frame, info->saved_regs, regnum,
-                              optimizedp, lvalp, addrp, realnump, bufferp);
+    trad_frame_get_prev_register (next_frame, info->saved_regs, regnum,
+				  optimizedp, lvalp, addrp, realnump, bufferp);
 }
 
 static const struct frame_unwind avr_frame_unwind = {
@@ -1297,8 +1297,6 @@ avr_gdbarch_init (struct gdbarch_info info, struct gdbarch_list *arches)
 
   set_gdbarch_breakpoint_from_pc (gdbarch, avr_breakpoint_from_pc);
 
-  set_gdbarch_deprecated_frameless_function_invocation (gdbarch, legacy_frameless_look_for_prologue);
-
   frame_unwind_append_sniffer (gdbarch, avr_frame_sniffer);
   frame_base_set_default (gdbarch, &avr_frame_base);
 
