@@ -44,11 +44,6 @@
 #define SOFTWARE_SINGLE_STEP_P() 1
 #define SOFTWARE_SINGLE_STEP(sig,bp_p) mips_software_single_step (sig, bp_p)
 
-/* FIXME: This still needs to be implemented.  */
-
-#undef  IN_SIGTRAMP
-#define IN_SIGTRAMP(pc, name)	(0)
-
 #undef IN_SOLIB_DYNSYM_RESOLVE_CODE
 #define IN_SOLIB_DYNSYM_RESOLVE_CODE(PC) mips_linux_in_dynsym_resolve_code (PC)
 int mips_linux_in_dynsym_resolve_code (CORE_ADDR pc);
