@@ -1,23 +1,25 @@
 /* Replay a remote debug session logfile for GDB.
-   Copyright (C) 1996 Free Software Foundation, Inc.
+   Copyright 1996, 1998, 1999, 2000, 2002, 2003 Free Software Foundation, Inc.
    Written by Fred Fish (fnf@cygnus.com) from pieces of gdbserver.
 
-This file is part of GDB.
+   This file is part of GDB.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.  */
 
+#include "config.h"
 #include <stdio.h>
 #include <sys/file.h>
 #include <netinet/in.h>
@@ -26,6 +28,18 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 #include <netinet/tcp.h>
 #include <signal.h>
 #include <ctype.h>
+#include <fcntl.h>
+#include <errno.h>
+
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
 /* Sort of a hack... */
 #define EOL (EOF - 1)
@@ -36,17 +50,19 @@ static int remote_desc;
    as the file name for which the error was encountered.
    Then return to command level.  */
 
-void
-perror_with_name (string)
-     char *string;
+static void
+perror_with_name (char *string)
 {
-  extern int sys_nerr;
-  extern char *sys_errlist[];
+#ifndef STDC_HEADERS
   extern int errno;
-  char *err;
+#endif
+  const char *err;
   char *combined;
 
-  err = (errno < sys_nerr) ? sys_errlist[errno] : "unknown error";
+  err = strerror (errno);
+  if (err == NULL)
+    err = "unknown error";
+
   combined = (char *) alloca (strlen (err) + strlen (string) + 3);
   strcpy (combined, string);
   strcat (combined, ": ");
@@ -57,11 +73,7 @@ perror_with_name (string)
 }
 
 static void
-sync_error (fp, desc, expect, got)
-     FILE *fp;
-     char *desc;
-     int expect;
-     int got;
+sync_error (FILE *fp, char *desc, int expect, int got)
 {
   fprintf (stderr, "\n%s\n", desc);
   fprintf (stderr, "At logfile offset %ld, expected '0x%x' got '0x%x'\n",
@@ -70,8 +82,8 @@ sync_error (fp, desc, expect, got)
   exit (1);
 }
 
-void
-remote_close()
+static void
+remote_close (void)
 {
   close (remote_desc);
 }
@@ -79,12 +91,9 @@ remote_close()
 /* Open a connection to a remote debugger.
    NAME is the filename used for communication.  */
 
-void
-remote_open (name)
-     char *name;
+static void
+remote_open (char *name)
 {
-  extern char *strchr ();
-
   if (!strchr (name, ':'))
     {
       fprintf (stderr, "%s: Must specify tcp connection as host:addr\n", name);
@@ -97,7 +106,6 @@ remote_open (name)
       int port;
       struct sockaddr_in sockaddr;
       int tmp;
-      struct protoent *protoent;
       int tmp_desc;
 
       port_str = strchr (name, ':');
@@ -110,40 +118,36 @@ remote_open (name)
 
       /* Allow rapid reuse of this port. */
       tmp = 1;
-      setsockopt (tmp_desc, SOL_SOCKET, SO_REUSEADDR, (char *)&tmp,
-		  sizeof(tmp));
+      setsockopt (tmp_desc, SOL_SOCKET, SO_REUSEADDR, (char *) &tmp,
+		  sizeof (tmp));
 
       sockaddr.sin_family = PF_INET;
-      sockaddr.sin_port = htons(port);
+      sockaddr.sin_port = htons (port);
       sockaddr.sin_addr.s_addr = INADDR_ANY;
 
-      if (bind (tmp_desc, (struct sockaddr *)&sockaddr, sizeof (sockaddr))
+      if (bind (tmp_desc, (struct sockaddr *) &sockaddr, sizeof (sockaddr))
 	  || listen (tmp_desc, 1))
 	perror_with_name ("Can't bind address");
 
       tmp = sizeof (sockaddr);
-      remote_desc = accept (tmp_desc, (struct sockaddr *)&sockaddr, &tmp);
+      remote_desc = accept (tmp_desc, (struct sockaddr *) &sockaddr, &tmp);
       if (remote_desc == -1)
 	perror_with_name ("Accept failed");
 
-      protoent = getprotobyname ("tcp");
-      if (!protoent)
-	perror_with_name ("getprotobyname");
-
       /* Enable TCP keep alive process. */
       tmp = 1;
-      setsockopt (tmp_desc, SOL_SOCKET, SO_KEEPALIVE, (char *)&tmp, sizeof(tmp));
+      setsockopt (tmp_desc, SOL_SOCKET, SO_KEEPALIVE, (char *) &tmp, sizeof (tmp));
 
       /* Tell TCP not to delay small packets.  This greatly speeds up
-	 interactive response. */
+         interactive response. */
       tmp = 1;
-      setsockopt (remote_desc, protoent->p_proto, TCP_NODELAY,
-		  (char *)&tmp, sizeof(tmp));
+      setsockopt (remote_desc, IPPROTO_TCP, TCP_NODELAY,
+		  (char *) &tmp, sizeof (tmp));
 
       close (tmp_desc);		/* No longer need this */
 
-      signal (SIGPIPE, SIG_IGN); /* If we don't do this, then gdbreplay simply
-				    exits when the remote side dies.  */
+      signal (SIGPIPE, SIG_IGN);	/* If we don't do this, then gdbreplay simply
+					   exits when the remote side dies.  */
     }
 
   fcntl (remote_desc, F_SETFL, FASYNC);
@@ -152,8 +156,8 @@ remote_open (name)
   fflush (stderr);
 }
 
-static int tohex (ch)
-     int ch;
+static int
+tohex (int ch)
 {
   if (ch >= '0' && ch <= '9')
     {
@@ -173,8 +177,7 @@ static int tohex (ch)
 }
 
 static int
-logchar (fp)
-     FILE *fp;
+logchar (FILE *fp)
 {
   int ch;
   int ch2;
@@ -193,13 +196,26 @@ logchar (fp)
       fflush (stdout);
       switch (ch)
 	{
-	case '\\': break;
-	case 'b': ch = '\b'; break;
-	case 'f': ch = '\f'; break;
-	case 'n': ch = '\n'; break;
-	case 'r': ch = '\r'; break;
-	case 't': ch = '\t'; break;
-	case 'v': ch = '\v'; break;
+	case '\\':
+	  break;
+	case 'b':
+	  ch = '\b';
+	  break;
+	case 'f':
+	  ch = '\f';
+	  break;
+	case 'n':
+	  ch = '\n';
+	  break;
+	case 'r':
+	  ch = '\r';
+	  break;
+	case 't':
+	  ch = '\t';
+	  break;
+	case 'v':
+	  ch = '\v';
+	  break;
 	case 'x':
 	  ch2 = fgetc (fp);
 	  fputc (ch2, stdout);
@@ -223,9 +239,8 @@ logchar (fp)
 /* Accept input from gdb and match with chars from fp (after skipping one
    blank) up until a \n is read from fp (which is not matched) */
 
-void
-expect (fp)
-     FILE *fp;
+static void
+expect (FILE *fp)
 {
   int fromlog;
   unsigned char fromgdb;
@@ -243,7 +258,8 @@ expect (fp)
 	  break;
 	}
       read (remote_desc, &fromgdb, 1);
-    } while (fromlog == fromgdb);
+    }
+  while (fromlog == fromgdb);
   if (fromlog != EOL)
     {
       sync_error (fp, "Sync error during read of gdb packet", fromlog,
@@ -254,9 +270,8 @@ expect (fp)
 /* Play data back to gdb from fp (after skipping leading blank) up until a
    \n is read from fp (which is discarded and not sent to gdb). */
 
-void
-play (fp)
-     FILE *fp;
+static void
+play (FILE *fp)
 {
   int fromlog;
   char ch;
@@ -274,9 +289,7 @@ play (fp)
 }
 
 int
-main (argc, argv)
-     int argc;
-     char *argv[];
+main (int argc, char *argv[])
 {
   FILE *fp;
   int ch;
@@ -291,7 +304,7 @@ main (argc, argv)
   if (fp == NULL)
     {
       perror_with_name (argv[1]);
-    }      
+    }
   remote_open (argv[2]);
   while ((ch = logchar (fp)) != EOF)
     {
@@ -314,4 +327,3 @@ main (argc, argv)
   remote_close ();
   exit (0);
 }
-
