@@ -73,13 +73,13 @@ i386nto_supply_gregset (char *gpregs)
   unsigned regno;
   int empty = 0;
 
-  for (regno = 0; regno < FP0_REGNUM; regno++)
+  for (regno = 0; regno < I386_NUM_GREGS; regno++)
     {
       int offset = nto_reg_offset (regno);
       if (offset == -1)
-	supply_register (regno, (char *) &empty);
+	regcache_raw_supply (current_regcache, regno, (char *) &empty);
       else
-	supply_register (regno, gpregs + offset);
+	regcache_raw_supply (current_regcache, regno, gpregs + offset);
     }
 }
 
@@ -111,9 +111,9 @@ i386nto_regset_id (int regno)
 {
   if (regno == -1)
     return NTO_REG_END;
-  else if (regno < FP0_REGNUM)
+  else if (regno < I386_NUM_GREGS)
     return NTO_REG_GENERAL;
-  else if (regno < FPC_REGNUM)
+  else if (regno < I386_NUM_GREGS + I386_NUM_FREGS)
     return NTO_REG_FLOAT;
 
   return -1;			/* Error.  */
@@ -176,7 +176,7 @@ i386nto_regset_fill (int regset, char *data)
 	{
 	  int offset = nto_reg_offset (regno);
 	  if (offset != -1)
-	    regcache_collect (regno, data + offset);
+	    regcache_raw_collect (current_regcache, regno, data + offset);
 	}
     }
   else if (regset == NTO_REG_FLOAT)
