@@ -45,9 +45,10 @@ mipsnbsd_supply_reg (char *regs, int regno)
       if (regno == i || regno == -1)
 	{
 	  if (CANNOT_FETCH_REGISTER (i))
-	    supply_register (i, NULL);
+	    regcache_raw_supply (current_regcache, i, NULL);
 	  else
-            supply_register (i, regs + (i * mips_isa_regsize (current_gdbarch)));
+            regcache_raw_supply (current_regcache, i,
+				 regs + (i * mips_isa_regsize (current_gdbarch)));
         }
     }
 }
@@ -59,7 +60,8 @@ mipsnbsd_fill_reg (char *regs, int regno)
 
   for (i = 0; i <= PC_REGNUM; i++)
     if ((regno == i || regno == -1) && ! CANNOT_STORE_REGISTER (i))
-      regcache_collect (i, regs + (i * mips_isa_regsize (current_gdbarch)));
+      regcache_raw_collect (current_regcache, i,
+			    regs + (i * mips_isa_regsize (current_gdbarch)));
 }
 
 void
@@ -74,9 +76,10 @@ mipsnbsd_supply_fpreg (char *fpregs, int regno)
       if (regno == i || regno == -1)
 	{
 	  if (CANNOT_FETCH_REGISTER (i))
-	    supply_register (i, NULL);
+	    regcache_raw_supply (current_regcache, i, NULL);
 	  else
-            supply_register (i, fpregs + ((i - FP0_REGNUM) * mips_isa_regsize (current_gdbarch)));
+            regcache_raw_supply (current_regcache, i,
+				 fpregs + ((i - FP0_REGNUM) * mips_isa_regsize (current_gdbarch)));
 	}
     }
 }
@@ -89,7 +92,8 @@ mipsnbsd_fill_fpreg (char *fpregs, int regno)
   for (i = FP0_REGNUM; i <= mips_regnum (current_gdbarch)->fp_control_status;
        i++)
     if ((regno == i || regno == -1) && ! CANNOT_STORE_REGISTER (i))
-      regcache_collect (i, fpregs + ((i - FP0_REGNUM) * mips_isa_regsize (current_gdbarch)));
+      regcache_raw_collect (current_regcache, i,
+			    fpregs + ((i - FP0_REGNUM) * mips_isa_regsize (current_gdbarch)));
 }
 
 static void
@@ -198,7 +202,7 @@ mipsnbsd_sigtramp_offset (CORE_ADDR pc)
   LONGEST off;
   int i;
 
-  if (read_memory_nobpt (pc, (char *) w, sizeof (w)) != 0)
+  if (deprecated_read_memory_nobpt (pc, (char *) w, sizeof (w)) != 0)
     return -1;
 
   for (i = 0; i < RETCODE_NWORDS; i++)
@@ -212,7 +216,7 @@ mipsnbsd_sigtramp_offset (CORE_ADDR pc)
   off = i * 4;
   pc -= off;
 
-  if (read_memory_nobpt (pc, (char *) ret, sizeof (ret)) != 0)
+  if (deprecated_read_memory_nobpt (pc, (char *) ret, sizeof (ret)) != 0)
     return -1;
 
   if (memcmp (ret, retcode, RETCODE_SIZE) == 0)
