@@ -1757,17 +1757,17 @@ mips_wait (ptid_t ptid, struct target_waitstatus *status)
     {
       char buf[MAX_REGISTER_SIZE];
 
-      store_unsigned_integer (buf, DEPRECATED_REGISTER_RAW_SIZE (PC_REGNUM), rpc);
-      supply_register (PC_REGNUM, buf);
+      store_unsigned_integer (buf, register_size (current_gdbarch, PC_REGNUM), rpc);
+      regcache_raw_supply (current_regcache, PC_REGNUM, buf);
 
-      store_unsigned_integer (buf, DEPRECATED_REGISTER_RAW_SIZE (PC_REGNUM), rfp);
-      supply_register (30, buf);	/* This register they are avoiding and so it is unnamed */
+      store_unsigned_integer (buf, register_size (current_gdbarch, PC_REGNUM), rfp);
+      regcache_raw_supply (current_regcache, 30, buf);	/* This register they are avoiding and so it is unnamed */
 
-      store_unsigned_integer (buf, DEPRECATED_REGISTER_RAW_SIZE (SP_REGNUM), rsp);
-      supply_register (SP_REGNUM, buf);
+      store_unsigned_integer (buf, register_size (current_gdbarch, SP_REGNUM), rsp);
+      regcache_raw_supply (current_regcache, SP_REGNUM, buf);
 
-      store_unsigned_integer (buf, DEPRECATED_REGISTER_RAW_SIZE (DEPRECATED_FP_REGNUM), 0);
-      supply_register (DEPRECATED_FP_REGNUM, buf);
+      store_unsigned_integer (buf, register_size (current_gdbarch, DEPRECATED_FP_REGNUM), 0);
+      regcache_raw_supply (current_regcache, DEPRECATED_FP_REGNUM, buf);
 
       if (nfields == 9)
 	{
@@ -1938,8 +1938,8 @@ mips_fetch_registers (int regno)
 
     /* We got the number the register holds, but gdb expects to see a
        value in the target byte ordering.  */
-    store_unsigned_integer (buf, DEPRECATED_REGISTER_RAW_SIZE (regno), val);
-    supply_register (regno, buf);
+    store_unsigned_integer (buf, register_size (current_gdbarch, regno), val);
+    regcache_raw_supply (current_regcache, regno, buf);
   }
 }
 
@@ -3306,7 +3306,7 @@ _initialize_remote_mips (void)
   mips_ops.to_fetch_registers = mips_fetch_registers;
   mips_ops.to_store_registers = mips_store_registers;
   mips_ops.to_prepare_to_store = mips_prepare_to_store;
-  mips_ops.to_xfer_memory = mips_xfer_memory;
+  mips_ops.deprecated_xfer_memory = mips_xfer_memory;
   mips_ops.to_files_info = mips_files_info;
   mips_ops.to_insert_breakpoint = mips_insert_breakpoint;
   mips_ops.to_remove_breakpoint = mips_remove_breakpoint;
@@ -3368,54 +3368,56 @@ of the TFTP temporary file, if it differs from the filename seen by the board.";
   add_target (&ddb_ops);
   add_target (&lsi_ops);
 
-  add_show_from_set (
-		      add_set_cmd ("timeout", no_class, var_zinteger,
-				   (char *) &mips_receive_wait,
-		       "Set timeout in seconds for remote MIPS serial I/O.",
-				   &setlist),
-		      &showlist);
+  deprecated_add_show_from_set
+    (add_set_cmd ("timeout", no_class, var_zinteger,
+		  (char *) &mips_receive_wait,
+		  "Set timeout in seconds for remote MIPS serial I/O.",
+		  &setlist),
+     &showlist);
 
-  add_show_from_set (
-		  add_set_cmd ("retransmit-timeout", no_class, var_zinteger,
-			       (char *) &mips_retransmit_wait,
-			       "Set retransmit timeout in seconds for remote MIPS serial I/O.\n\
+  deprecated_add_show_from_set
+    (add_set_cmd ("retransmit-timeout", no_class, var_zinteger,
+		  (char *) &mips_retransmit_wait, "\
+Set retransmit timeout in seconds for remote MIPS serial I/O.\n\
 This is the number of seconds to wait for an acknowledgement to a packet\n\
 before resending the packet.", &setlist),
-		      &showlist);
+     &showlist);
 
-  add_show_from_set (
-		   add_set_cmd ("syn-garbage-limit", no_class, var_zinteger,
-				(char *) &mips_syn_garbage,
-				"Set the maximum number of characters to ignore when scanning for a SYN.\n\
+  deprecated_add_show_from_set
+    (add_set_cmd ("syn-garbage-limit", no_class, var_zinteger,
+		  (char *) &mips_syn_garbage, "\
+Set the maximum number of characters to ignore when scanning for a SYN.\n\
 This is the maximum number of characters GDB will ignore when trying to\n\
-synchronize with the remote system.  A value of -1 means that there is no limit\n\
-(Note that these characters are printed out even though they are ignored.)",
-				&setlist),
-		      &showlist);
+synchronize with the remote system.  A value of -1 means that there is no\n\
+limit. (Note that these characters are printed out even though they are\n\
+ignored.)",
+		  &setlist),
+     &showlist);
 
-  add_show_from_set
+  deprecated_add_show_from_set
     (add_set_cmd ("monitor-prompt", class_obscure, var_string,
 		  (char *) &mips_monitor_prompt,
 		  "Set the prompt that GDB expects from the monitor.",
 		  &setlist),
      &showlist);
 
-  add_show_from_set (
-	       add_set_cmd ("monitor-warnings", class_obscure, var_zinteger,
-			    (char *) &monitor_warnings,
-			    "Set printing of monitor warnings.\n"
-		"When enabled, monitor warnings about hardware breakpoints "
-			    "will be displayed.",
-			    &setlist),
-		      &showlist);
+  deprecated_add_show_from_set
+    (add_set_cmd ("monitor-warnings", class_obscure, var_zinteger,
+		  (char *) &monitor_warnings,
+		  "Set printing of monitor warnings.\n"
+		  "When enabled, monitor warnings about hardware breakpoints "
+		  "will be displayed.",
+		  &setlist),
+     &showlist);
 
   add_com ("pmon <command>", class_obscure, pmon_command,
 	   "Send a packet to PMON (must be in debug mode).");
 
-  add_show_from_set (add_set_cmd ("mask-address", no_class,
-				  var_boolean, &mask_address_p,
-				  "Set zeroing of upper 32 bits of 64-bit addresses when talking to PMON targets.\n\
+  deprecated_add_show_from_set
+    (add_set_cmd ("mask-address", no_class,
+		  var_boolean, &mask_address_p, "\
+Set zeroing of upper 32 bits of 64-bit addresses when talking to PMON targets.\n\
 Use \"on\" to enable the masking and \"off\" to disable it.\n",
-				  &setlist),
-		     &showlist);
+		  &setlist),
+     &showlist);
 }
