@@ -216,7 +216,8 @@ som_solib_sizeof_symbol_table (char *filename)
   /* We believe that filename was handed to us by the dynamic linker, and
      is therefore always an absolute path.
    */
-  desc = openp (getenv ("PATH"), 1, filename, O_RDONLY | O_BINARY, 0, &absolute_name);
+  desc = openp (getenv ("PATH"), OPF_TRY_CWD_FIRST, filename,
+		O_RDONLY | O_BINARY, 0, &absolute_name);
   if (desc < 0)
     {
       perror_with_name (filename);
@@ -1462,17 +1463,17 @@ som_sharedlibrary_info_command (char *ignore, int from_tty)
       if (so_list->objfile == NULL)
 	printf_unfiltered ("  (symbols not loaded)");
       printf_unfiltered ("\n");
-      printf_unfiltered ("    %-12s", local_hex_string_custom (flags, "08l"));
+      printf_unfiltered ("    %-12s", hex_string_custom (flags, 8));
       printf_unfiltered ("%-12s",
-	     local_hex_string_custom (so_list->som_solib.text_addr, "08l"));
+			 hex_string_custom (so_list->som_solib.text_addr, 8));
       printf_unfiltered ("%-12s",
-	      local_hex_string_custom (so_list->som_solib.text_end, "08l"));
+			 hex_string_custom (so_list->som_solib.text_end, 8));
       printf_unfiltered ("%-12s",
-	    local_hex_string_custom (so_list->som_solib.data_start, "08l"));
+			 hex_string_custom (so_list->som_solib.data_start, 8));
       printf_unfiltered ("%-12s",
-	      local_hex_string_custom (so_list->som_solib.data_end, "08l"));
+			 hex_string_custom (so_list->som_solib.data_end, 8));
       printf_unfiltered ("%-12s\n",
-	     local_hex_string_custom (so_list->som_solib.got_value, "08l"));
+			 hex_string_custom (so_list->som_solib.got_value, 8));
       so_list = so_list->next;
     }
 }
@@ -1583,7 +1584,7 @@ _initialize_som_solib (void)
   add_info ("sharedlibrary", som_sharedlibrary_info_command,
 	    "Status of loaded shared object libraries.");
 
-  add_show_from_set
+  deprecated_add_show_from_set
     (add_set_cmd ("auto-solib-add", class_support, var_boolean,
 		  (char *) &auto_solib_add,
 		  "Set autoloading of shared library symbols.\n\
@@ -1594,7 +1595,7 @@ inferior.  Otherwise, symbols must be loaded manually, using `sharedlibrary'.",
 		  &setlist),
      &showlist);
 
-  add_show_from_set
+  deprecated_add_show_from_set
     (add_set_cmd ("auto-solib-limit", class_support, var_zinteger,
 		  (char *) &auto_solib_limit,
 		  "Set threshold (in Mb) for autoloading shared library symbols.\n\
