@@ -1,6 +1,6 @@
 /* Target-dependent code for GNU/Linux SPARC.
 
-   Copyright 2003 Free Software Foundation, Inc.
+   Copyright 2003, 2004 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -83,10 +83,12 @@
    Otherwise, return 0.  */
 
 CORE_ADDR
-sparc_linux_sigtramp_start (CORE_ADDR pc, ULONGEST insn0, ULONGEST insn1)
+sparc_linux_sigtramp_start (struct frame_info *next_frame,
+			    ULONGEST insn0, ULONGEST insn1)
 {
+  CORE_ADDR pc = frame_pc_unwind (next_frame);
   ULONGEST word0, word1;
-  char buf[8];			/* Two instructions.  */
+  unsigned char buf[8];		/* Two instructions.  */
 
   /* We only recognize a signal trampoline if PC is at the start of
      one of the instructions.  We optimize for finding the PC at the
@@ -96,7 +98,7 @@ sparc_linux_sigtramp_start (CORE_ADDR pc, ULONGEST insn0, ULONGEST insn1)
      sequence, there will be a few trailing readable bytes on the
      stack.  */
 
-  if (read_memory_nobpt (pc, buf, sizeof buf) != 0)
+  if (!safe_frame_unwind_memory (next_frame, pc, buf, sizeof buf))
     return 0;
 
   word0 = extract_unsigned_integer (buf, 4);
@@ -106,7 +108,7 @@ sparc_linux_sigtramp_start (CORE_ADDR pc, ULONGEST insn0, ULONGEST insn1)
 	return 0;
 
       pc -= 4;
-      if (read_memory_nobpt (pc, buf, sizeof buf) != 0)
+      if (!safe_frame_unwind_memory (next_frame, pc, buf, sizeof buf))
 	return 0;
 
       word0 = extract_unsigned_integer (buf, 4);
@@ -120,30 +122,35 @@ sparc_linux_sigtramp_start (CORE_ADDR pc, ULONGEST insn0, ULONGEST insn1)
 }
 
 static CORE_ADDR
-sparc32_linux_sigtramp_start (CORE_ADDR pc)
+sparc32_linux_sigtramp_start (struct frame_info *next_frame)
 {
-  return sparc_linux_sigtramp_start (pc, LINUX32_SIGTRAMP_INSN0,
+  return sparc_linux_sigtramp_start (next_frame, LINUX32_SIGTRAMP_INSN0,
 				     LINUX32_SIGTRAMP_INSN1);
 }
 
 static CORE_ADDR
-sparc32_linux_rt_sigtramp_start (CORE_ADDR pc)
+sparc32_linux_rt_sigtramp_start (struct frame_info *next_frame)
 {
-  return sparc_linux_sigtramp_start (pc, LINUX32_RT_SIGTRAMP_INSN0,
+  return sparc_linux_sigtramp_start (next_frame, LINUX32_RT_SIGTRAMP_INSN0,
 				     LINUX32_RT_SIGTRAMP_INSN1);
 }
 
 static int
-sparc32_linux_pc_in_sigtramp (CORE_ADDR pc, char *name)
+sparc32_linux_sigtramp_p (struct frame_info *next_frame)
 {
+  CORE_ADDR pc = frame_pc_unwind (next_frame);
+  char *name;
+
+  find_pc_partial_function (pc, &name, NULL, NULL);
+
   /* If we have NAME, we can optimize the search.  The trampolines are
      named __restore and __restore_rt.  However, they aren't dynamically
      exported from the shared C library, so the trampoline may appear to
      be part of the preceding function.  This should always be sigaction,
      __sigaction, or __libc_sigaction (all aliases to the same function).  */
   if (name == NULL || strstr (name, "sigaction") != NULL)
-    return (sparc32_linux_sigtramp_start (pc) != 0
-	    || sparc32_linux_rt_sigtramp_start (pc) != 0);
+    return (sparc32_linux_sigtramp_start (next_frame) != 0
+	    || sparc32_linux_rt_sigtramp_start (next_frame) != 0);
 
   return (strcmp ("__restore", name) == 0
 	  || strcmp ("__restore_rt", name) == 0);
@@ -170,13 +177,12 @@ sparc32_linux_sigtramp_frame_cache (struct frame_info *next_frame,
   regnum = SPARC_O1_REGNUM;
   sigcontext_addr = frame_unwind_register_unsigned (next_frame, regnum);
 
-  cache->pc = frame_pc_unwind (next_frame);
-  addr = sparc32_linux_sigtramp_start (cache->pc);
+  addr = sparc32_linux_sigtramp_start (next_frame);
   if (addr == 0)
     {
       /* If this is a RT signal trampoline, adjust SIGCONTEXT_ADDR
          accordingly.  */
-      addr = sparc32_linux_rt_sigtramp_start (cache->pc);
+      addr = sparc32_linux_rt_sigtramp_start (next_frame);
       if (addr)
 	sigcontext_addr += 128;
       else
@@ -225,8 +231,8 @@ sparc32_linux_sigtramp_frame_prev_register (struct frame_info *next_frame,
   struct sparc_frame_cache *cache =
     sparc32_linux_sigtramp_frame_cache (next_frame, this_cache);
 
-  trad_frame_prev_register (next_frame, cache->saved_regs, regnum,
-			    optimizedp, lvalp, addrp, realnump, valuep);
+  trad_frame_get_prev_register (next_frame, cache->saved_regs, regnum,
+				optimizedp, lvalp, addrp, realnump, valuep);
 }
 
 static const struct frame_unwind sparc32_linux_sigtramp_frame_unwind =
@@ -239,47 +245,13 @@ static const struct frame_unwind sparc32_linux_sigtramp_frame_unwind =
 static const struct frame_unwind *
 sparc32_linux_sigtramp_frame_sniffer (struct frame_info *next_frame)
 {
-  CORE_ADDR pc = frame_pc_unwind (next_frame);
-  char *name;
-
-  find_pc_partial_function (pc, &name, NULL, NULL);
-  if (sparc32_linux_pc_in_sigtramp (pc, name))
+  if (sparc32_linux_sigtramp_p (next_frame))
     return &sparc32_linux_sigtramp_frame_unwind;
 
   return NULL;
 }
 
 
-static struct link_map_offsets *
-sparc32_linux_svr4_fetch_link_map_offsets (void)
-{
-  static struct link_map_offsets lmo;
-  static struct link_map_offsets *lmp = NULL;
-
-  if (lmp == NULL)
-    {
-      lmp = &lmo;
-
-      /* Everything we need is in the first 8 bytes.  */
-      lmo.r_debug_size = 8;
-      lmo.r_map_offset = 4;
-      lmo.r_map_size   = 4;
-
-      /* Everything we need is in the first 20 bytes.  */
-      lmo.link_map_size = 20;
-      lmo.l_addr_offset = 0;
-      lmo.l_addr_size   = 4;
-      lmo.l_name_offset = 4;
-      lmo.l_name_size   = 4;
-      lmo.l_next_offset = 12;
-      lmo.l_next_size   = 4;
-      lmo.l_prev_offset = 16;
-      lmo.l_prev_size   = 4;
-    }
-
-  return lmp;
-}
-
 static void
 sparc32_linux_init_abi (struct gdbarch_info info, struct gdbarch *gdbarch)
 {
@@ -294,9 +266,6 @@ sparc32_linux_init_abi (struct gdbarch_info info, struct gdbarch *gdbarch)
   set_gdbarch_long_double_format (gdbarch, &floatformat_ieee_double_big);
 
   frame_unwind_append_sniffer (gdbarch, sparc32_linux_sigtramp_frame_sniffer);
-
-  set_solib_svr4_fetch_link_map_offsets
-    (gdbarch, sparc32_linux_svr4_fetch_link_map_offsets);
 }
 
 /* Provide a prototype to silence -Wmissing-prototypes.  */
