@@ -169,8 +169,8 @@ sparc64fbsd_sigtramp_frame_prev_register (struct frame_info *next_frame,
   struct sparc_frame_cache *cache =
     sparc64fbsd_sigtramp_frame_cache (next_frame, this_cache);
 
-  trad_frame_prev_register (next_frame, cache->saved_regs, regnum,
-			    optimizedp, lvalp, addrp, realnump, valuep);
+  trad_frame_get_prev_register (next_frame, cache->saved_regs, regnum,
+				optimizedp, lvalp, addrp, realnump, valuep);
 }
 
 static const struct frame_unwind sparc64fbsd_sigtramp_frame_unwind =
