@@ -97,6 +97,14 @@ trad_frame_set_value (struct trad_frame_saved_reg this_saved_regs[],
   this_saved_regs[regnum].addr = val;
 }
 
+void
+trad_frame_set_reg_realreg (struct trad_frame_cache *this_trad_cache,
+			    int regnum, int realreg)
+{
+  this_trad_cache->prev_regs[regnum].realreg = realreg;
+  this_trad_cache->prev_regs[regnum].addr = -1;
+}
+
 void
 trad_frame_set_reg_addr (struct trad_frame_cache *this_trad_cache,
 			 int regnum, CORE_ADDR addr)
@@ -114,11 +122,11 @@ trad_frame_set_unknown (struct trad_frame_saved_reg this_saved_regs[],
 }
 
 void
-trad_frame_prev_register (struct frame_info *next_frame,
-			  struct trad_frame_saved_reg this_saved_regs[],
-			  int regnum, int *optimizedp,
-			  enum lval_type *lvalp, CORE_ADDR *addrp,
-			  int *realregp, void *bufferp)
+trad_frame_get_prev_register (struct frame_info *next_frame,
+			      struct trad_frame_saved_reg this_saved_regs[],
+			      int regnum, int *optimizedp,
+			      enum lval_type *lvalp, CORE_ADDR *addrp,
+			      int *realregp, void *bufferp)
 {
   struct gdbarch *gdbarch = get_frame_arch (next_frame);
   if (trad_frame_addr_p (this_saved_regs, regnum))
@@ -166,9 +174,9 @@ trad_frame_get_register (struct trad_frame_cache *this_trad_cache,
 			 enum lval_type *lvalp, CORE_ADDR *addrp,
 			 int *realregp, void *bufferp)
 {
-  trad_frame_prev_register (next_frame, this_trad_cache->prev_regs,
-			    regnum, optimizedp, lvalp, addrp, realregp,
-			    bufferp);
+  trad_frame_get_prev_register (next_frame, this_trad_cache->prev_regs,
+				regnum, optimizedp, lvalp, addrp, realregp,
+				bufferp);
 }
 
 void
@@ -184,3 +192,16 @@ trad_frame_get_id (struct trad_frame_cache *this_trad_cache,
 {
   (*this_id) = this_trad_cache->this_id;
 }
+
+void
+trad_frame_set_this_base (struct trad_frame_cache *this_trad_cache,
+			  CORE_ADDR this_base)
+{
+  this_trad_cache->this_base = this_base;
+}
+
+CORE_ADDR
+trad_frame_get_this_base (struct trad_frame_cache *this_trad_cache)
+{
+  return this_trad_cache->this_base;
+}
