@@ -532,8 +532,8 @@ v850ice_fetch_registers (int regno)
     error ("v850ice_fetch_registers (%d):  bad value from ICE: %s.",
 	   regno, val);
 
-  store_unsigned_integer (val, DEPRECATED_REGISTER_RAW_SIZE (regno), regval);
-  supply_register (regno, val);
+  store_unsigned_integer (val, register_size (current_gdbarch, regno), regval);
+  regcache_raw_supply (current_regcache, regno, val);
 }
 
 /* Store register REGNO, or all registers if REGNO == -1, from the contents
@@ -558,7 +558,7 @@ v850ice_store_registers (int regno)
     }
 
   regval = extract_unsigned_integer (&deprecated_registers[DEPRECATED_REGISTER_BYTE (regno)],
-				     DEPRECATED_REGISTER_RAW_SIZE (regno));
+				     register_size (current_gdbarch, regno));
   strcpy (cmd, "reg ");
   if (!convert_register (regno, &cmd[4]))
     return;
@@ -899,7 +899,7 @@ init_850ice_ops (void)
   v850ice_ops.to_fetch_registers = v850ice_fetch_registers;
   v850ice_ops.to_store_registers = v850ice_store_registers;
   v850ice_ops.to_prepare_to_store = v850ice_prepare_to_store;
-  v850ice_ops.to_xfer_memory = v850ice_xfer_memory;
+  v850ice_ops.deprecated_xfer_memory = v850ice_xfer_memory;
   v850ice_ops.to_files_info = v850ice_files_info;
   v850ice_ops.to_insert_breakpoint = v850ice_insert_breakpoint;
   v850ice_ops.to_remove_breakpoint = v850ice_remove_breakpoint;
